// ___SETA43  _____
// 27/03/2025 _____
// RADIO SI4732
// mando ROTARY ENCODER v2
// Arduino 1.8.19
// seta43.duckdns.org
// seta43.blogspot.com

/*
    | ROTARY ENCODER  | ARDUINO
    | ----------------| -----------  |
    | IZQUIERDA       |     2        |
    | DERECHA         |     3        |
    | BUTTON          |     4        |
 */

/*
    | Si4732 pin      |  Arduino Pin  |
    | ----------------| ------------  |
    | RESET (pin 9)   |     12        |
    | SDIO (pin 12)   |     A4        |
    | CLK (pin 11)    |     A5        |
*/

char nombre[]="\nSETA43 SI4732";

#include "Rotary.h";
#define ROTARY_PIN1  2
#define ROTARY_PIN2 3
#define BUTTON_PIN  4
#define CLICKS_PER_STEP   2   // this number depends on your rotary encoder 

/////////////////////////////////////////////////////////////////
//___________OLED_________
#include "olex.h"
#include <Wire.h>
/////////////////////////////////////////////////////////////////
Rotary r;
int rNum=10;
int rDir;
/////////////////////////////////////////////////////////////////
#define FRE_MIN 8700
#define FRE_MAX 10800
uint16_t frecuOld,frecuencia=8800;
int iS,iN,iStereo;
/////////////////////////////////////////////////////////////////
#include <SI4735.h>
#define RESET_PIN 12
#define AM_FUNCTION 1
#define FM_FUNCTION 0
SI4735 chipRadio;
/////////////////////////////////////////////////////////////////
char cadena[40];

int t;

void setup() 
{
  Serial.begin(9600);
  while (!Serial) {;}
  Serial.println(nombre);
  pinMode(RESET_PIN, OUTPUT);  
  digitalWrite(RESET_PIN, HIGH);
  delay(200);  
  Wire.begin(); // I2C bus
  scanI2C();
  OleInit();
  delay(200);
  xclearDisplay();  ModeFont=0; xStringAT(0,0,(char *)nombre);
  delay(300); xclearDisplay(); 
 
  r.begin(ROTARY_PIN1, ROTARY_PIN2, CLICKS_PER_STEP);
  r.setLeftRotationHandler(showDirection);
  r.setRightRotationHandler(showDirection);

  pinMode(BUTTON_PIN, INPUT_PULLUP); 

  int16_t si4735Addr = chipRadio.getDeviceI2CAddress(RESET_PIN);
  if ( si4735Addr == 0 ) {Serial.println("Si4732 not found!"); while (1) ;} 
  delay(100);
  chipRadio.setup(RESET_PIN, FM_FUNCTION);
  chipRadio.setFM(8400, 10800, frecuencia, 10);  
  chipRadio.setVolume(63); //Sets volume level (0 to 63 max). 
  chipRadio.setRdsConfig(3, 3, 3, 3, 3);
  chipRadio.setFifoCount(1);
  rNum=(frecuencia-FRE_MIN)/10;
  Serial.println(rNum); Serial.println(frecuencia);
  delay(200);
  printFrecuOLED();
  chipRadio.setFrequency(frecuencia);
}

void loop() 
{
    delay(1);
    t++; 
    if(t>1000)
        {
          t=0;  
          if(frecuencia!=chipRadio.getFrequency()) {chipRadio.setFrequency(frecuencia); Serial.print("*"); }    
          chipRadio.getCurrentReceivedSignalQuality();
          iStereo=chipRadio.getCurrentPilot();
          iN=chipRadio.getCurrentSNR();
          iS=chipRadio.getCurrentRSSI();
          printDataOLED();
          checkRds(); Serial.println("<RDS");
        }
    r.loop();
    if (digitalRead(BUTTON_PIN) == 0) 
        {     
          delay(100);
          while(digitalRead(BUTTON_PIN) == 0) ;
          Serial.println("BOTON");
        }
}

//=======================================
void printDataOLED(void)
{
  ModeFont=0;
  if(iStereo==0) xStringAT(16,1,"Mono"); else xStringAT(16,1," >< ");
  sprintf(cadena,"SNR:%02u SIG:%02u ",iN,iS); xStringAT(0,5,(char *)cadena);
  Serial.print(cadena);
  sprintf(cadena,"%03d.%02d MHz",frecuencia/100,frecuencia%100);
  Serial.print(cadena);
  if(iStereo==0) Serial.print(" Mono   "); else Serial.print(" Stereo ");
}

void printFrecuOLED(void)
{
  ModeFont=2; 
  sprintf(cadena,"%03d.%02d",frecuencia/100,frecuencia%100);
  xStringAT(0,0,(char *)cadena);
  ModeFont=0; xStringAT(12,1,"MHz");
}


void showDirection(Rotary& r) 
{
  if(r.getDirection()==1) 
      { rDir=-1; chipRadio.frequencyDown();} 
   else
      { rDir=1; chipRadio.frequencyUp();}
  t=0;
  frecuencia=chipRadio.getFrequency();
  printFrecuOLED();
}



void checkRds() 
{
 int z,t;
 char c;
 char *stationName;
 char miSDR[15];
 
  chipRadio.rdsBeginQuery();
  stationName = chipRadio.getRdsStationName();
  z=chipRadio.getNumRdsFifoUsed();
  ModeFont=0; 
  if ( stationName != NULL ) 
  {   
    if(z>2)
      {
        Serial.print(">>");
        for(t=0;t<z && t<12;t++)
        {
          c=*(stationName+t);
          if(c==0 || (c>31 && c<126))  {miSDR[t]=c;}        
          xCaracterAT(t,3, c); 
          Serial.print(c);
        }   
      }
  }
}




//=================scanI2C=============
void scanI2C() 
{
 int nDevices = 0;

  Serial.println(F("Scanning..."));
  for (byte address = 1; address < 127; ++address) 
  {
    Wire.beginTransmission(address);
    byte error = Wire.endTransmission();

    if (error == 0) 
      {
            Serial.print(nDevices);
            Serial.print(F("-I2C device found at address 0x"));
            if (address < 16) {   Serial.print("0");  }
            Serial.print(address, HEX); 
            switch(address)
            {
              case 0x11:
                Serial.println(F(" SI473X / SI474X"));
              break;
              case 0x38:
                Serial.println(F(" AHTxx"));
              break;
              case 0x3C:
                Serial.println(F(" OLED"));
              break;
              case 0x50:
                Serial.println(F(" AT24C32"));
              break;
               case 0x68:
                Serial.println(F(" DS1371"));
              break;
               case 0x77:
                Serial.println(F(" BMP280"));
              break;
              default:
              Serial.println(F(" ¿Desconocido?"));
              break;
            }
      
            ++nDevices;
          } 
          else 
            if (error == 4) 
            {
            Serial.print(F("Unknown error at address 0x"));
            if (address < 16) {  Serial.print("0"); }
            Serial.println(address, HEX);
            }
    }
    if (nDevices == 0) 
        {  Serial.println(F("No I2C devices found\n")); } 
      else 
        {  Serial.println(F("done\n"));  }
}
